Package: EFSATools
Type: Package
Title: EFSA Ensemble of Data Collections Tools
Version: 1.0.0
Maintainer: Luca Belmonte <luca.belmonte@efsa.europa.eu>
Authors@R: 
  c(person(given = "Lorenzo",
           family = "Copelli",
           role = "aut",
           comment = c(ORCID = "0009-0002-4305-065X")),
    person(given = "Luca",
           family = "Belmonte",
           role = c("aut", "cre"),
           email = "luca.belmonte@efsa.europa.eu",
           comment = c(ORCID = "0000-0002-7977-9170")))
Description: Provides tools for dataset operations and utilities designed to
  preserve data history within EFSA's ad hoc data collections. It also
  imports packages developed by EFSA that provide additional support for data
  collection activities.
License: EUPL-1.2
URL: https://openefsa.github.io/EFSATools/
BugReports: https://github.com/openefsa/EFSATools/issues
Depends: R (>= 4.1.0)
Imports: checkmate (>= 2.3.1), dplyr (>= 1.1.4), glue (>= 1.7.0), rlang
        (>= 1.1.4), stringr (>= 1.5.1), tidyr (>= 1.3.1), eppoFindeR
        (>= 2.0.0), distilleR (>= 1.0.0)
Suggests: devtools (>= 2.4.5), tibble (>= 3.3.0), covr (>= 3.6.4),
        knitr (>= 1.0), rmarkdown (>= 2.0), testthat (>= 3.0.0),
        usethis (>= 2.2.3), roxygen2 (>= 7.2.1), cli (>= 3.6.5)
Encoding: UTF-8
RoxygenNote: 7.3.3
Config/testthat/edition: 3
VignetteBuilder: knitr
Config/Needs/website: pkgdown
Repository: CRAN
NeedsCompilation: no
Packaged: 2026-04-27 10:05:51 UTC; copello
Author: Lorenzo Copelli [aut] (ORCID: <https://orcid.org/0009-0002-4305-065X>),
  Luca Belmonte [aut, cre] (ORCID:
    <https://orcid.org/0000-0002-7977-9170>)
Date/Publication: 2026-04-28 20:00:08 UTC
Built: R 4.6.0; ; 2026-05-01 21:56:41 UTC; unix

# Generated by roxygen2: do not edit by hand

export(SCD2)
export(SSCD2)
export(dropEmpty)
export(enrich)
export(removeReplicatedColumns)
import(distilleR)
import(eppoFindeR)
importFrom(checkmate,assert_data_frame)
importFrom(checkmate,assert_names)
importFrom(checkmate,assert_string)
importFrom(checkmate,assert_vector)
importFrom(dplyr,across)
importFrom(dplyr,anti_join)
importFrom(dplyr,any_of)
importFrom(dplyr,bind_cols)
importFrom(dplyr,bind_rows)
importFrom(dplyr,filter)
importFrom(dplyr,inner_join)
importFrom(dplyr,left_join)
importFrom(dplyr,mutate)
importFrom(dplyr,mutate_all)
importFrom(dplyr,na_if)
importFrom(dplyr,right_join)
importFrom(dplyr,select)
importFrom(dplyr,starts_with)
importFrom(dplyr,where)
importFrom(glue,glue)
importFrom(rlang,":=")
importFrom(rlang,.data)
importFrom(rlang,sym)
importFrom(stringr,regex)
importFrom(stringr,str_replace_all)
importFrom(tidyr,unite)

#' Implement a "Simple" Slowly Changing Dimension Type 2.
#' 
#' This function implements a Simplified version of Slowly Changing Dimension
#' Type 2 to merge new and current data while maintaining historical records.
#' The function deactivates all the old records and activates new ones, ensuring
#' a history-preserving update strategy. The difference between a standard SCD2
#' is that this simplified version applies no checks on the data, deactivating
#' all the old records and activating the new ones, even if some of the old
#' records are still active.
#' 
#' @param newData `data.frame`. The data frame containing new records.
#' 
#' @param currentData `data.frame`. The data frame containing existing records.
#' 
#' @return A combined data frame with all old data marked as not active and new
#'   data marked as active.
#' 
#' @importFrom checkmate assert_data_frame
#' @importFrom dplyr bind_rows
#' 
#' @examples
#' currentData_ <- tibble::tribble(
#'   ~id, ~colA, ~colB, ~colC, ~IS_ACTIVE, ~START_DATE, ~END_DATE,
#'   1, "a1", "b1", "c1", TRUE, Sys.time(), as.Date(NA),
#'   2, "a2", "b2", "c2", TRUE, Sys.time(), as.Date(NA),
#'   3, "a3", "b3", "c3", TRUE, Sys.time(), as.Date(NA))
#' 
#' newData_ <- tibble::tribble(
#'   ~id, ~colA, ~colB, ~colC,
#'   1, "a1", "b1", "c1",
#'   2, "a2", "b2", "c20",
#'   3, "a4", "b4", "c4")
#' 
#' mergedData <- SSCD2(newData = newData_, currentData = currentData_)
#' 
#' @export
#' 
SSCD2 <- function(newData, currentData) {

  assert_data_frame(newData)
  assert_data_frame(currentData)

  mergedData_ <- bind_rows(
    .deactivate(currentData),
    .activate(newData))

  return(mergedData_)
}

test_that("The new data must be a data frame", {
  expect_error(
    SSCD2(
      newData = 1,
      currentData = tibble())
  )
})

test_that("The current data must be a data frame", {
  expect_error(
    SSCD2(
      newData = tibble(),
      currentData = 1)
  )
})

test_that("The function must output as expected", {
  currentData_ <- tibble::tribble(
    ~id, ~colA, ~colB, ~colC,
    1, "a1", "b1", "c1",
    2, "a2", "b2", "c2",
    3, "a3", "b3", "c3"
  )
  currentData_ <- .activate(dataframe = currentData_)

  newData_ <- tibble::tribble(
    ~id, ~colA, ~colB, ~colC,
    1, "a1", "b1", "c1",  # Identical row.
    2, "a2", "b2", "c20", # Almost identical row.
    3, "a4", "b4", "c4"   # Different row.
  )

  mergedData_ <- SSCD2(newData = newData_, currentData = currentData_)

  expectedResult_ <- tibble::tribble(
    ~id, ~colA, ~colB, ~colC, ~IS_ACTIVE,
    1, "a1", "b1", "c1", FALSE,
    2, "a2", "b2", "c2", FALSE,
    3, "a3", "b3", "c3", FALSE,
    1, "a1", "b1", "c1", TRUE,
    2, "a2", "b2", "c20", TRUE,
    3, "a4", "b4", "c4", TRUE) |>
    dplyr::arrange(id)

  arrangedMergedData_ <- mergedData_ |>
    dplyr::select(-ends_with("DATE")) |>
    dplyr::arrange(id)

  expect_equal(expectedResult_, arrangedMergedData_)
})

#' Drop and merge replicated columns from the specified data frame.
#' 
#' This function drops and merges all the replicated columns from the specified
#' data frame.
#'
#' @param dataframe `data.frame`. The data frame from which to drop the
#'   replicated columns.
#' 
#' @param prefix `character` (string). The prefix with which the name of the
#'   replicated columns starts.
#' 
#' @returns The specified data frame with an additional deduplicated column and
#'   all the types transformed to string.
#'
#' @importFrom dplyr mutate across starts_with select bind_cols na_if any_of
#' @importFrom tidyr unite
#' @importFrom stringr str_replace_all regex
#' @importFrom glue glue
#' @importFrom checkmate assert_data_frame assert_string
#' @importFrom rlang := sym
#' 
#' @details
#' All the occurrences of "N/A", "NA", and empty strings (case insensitive)
#' inside the provided data frame are replaced with NAs of type character.
#' Then, all and only the columns starting with the specified prefix are
#' selected and united into a single column with name ending per
#' "_deduplicated". All empty entries in the new deduplicated column are
#' replaced with NAs. Finally, the new column is bound with the other columns of
#' the initial dataframe.
#'
#' @examples
#' irisTest_ <- iris
#' irisTest_$Species_1 <- irisTest_$Species
#' irisTest_$Species_2 <- irisTest_$Species
#' irisTest_$Species <- NULL
#' 
#' deduplicatedDataframe_ <- removeReplicatedColumns(
#'   dataframe = irisTest_,
#'   prefix = "Species_")
#'
#' @export
#'
removeReplicatedColumns <- function(dataframe, prefix) {

  assert_data_frame(dataframe)
  assert_string(prefix)

  dataframe <- dataframe |>
    
    mutate(across(starts_with(prefix), as.character)) |>
    
    mutate(across(starts_with(prefix),
      \(x) str_replace_all(
        x,
        regex("^N/a$|^Na$|^$", ignore_case = TRUE),
        NA_character_)))

  dataframe_prefix_columns_ <- dataframe |>
    select(starts_with(prefix))

  dataframe_prefix_columns_ <- dataframe_prefix_columns_ |>
    
    unite(!!glue("{prefix}_deduplicated"),
      1:ncol(dataframe_prefix_columns_),
      sep = "",
      na.rm = TRUE) |>
    
    mutate(!!glue("{prefix}_deduplicated")
      := na_if(!!sym(glue("{prefix}_deduplicated")), "")) |>
    
    bind_cols(
      dataframe |>
      select(-starts_with(prefix)) |>
      select(any_of(names(dataframe)))) |>
    
    mutate_all(as.character)

  return(dataframe_prefix_columns_)
}

test_that("The data frame to manipulate must be a data frame", {
  expect_error(
    removeReplicatedColumns(
      dataframe = 1,
      prefix = "")
  )
})

test_that("The prefix must be a string", {
  expect_error(
    removeReplicatedColumns(
      dataframe = tibble(),
      prefix = 1)
  )
})

test_that("A data frame must be returned", {
  dataframe_ <- data.frame(
    prefix_1 = c(1, NA, NA),
    prefix_2 = c(NA, 2, NA),
    prefix_3 = c(NA, NA, 3),
    another_col = c(NA, NA, 3))

  dataframeDeduplicated_ <- suppressWarnings(
    removeReplicatedColumns(
      dataframe = dataframe_,
      prefix = "prefix"))

  expect_true(is.data.frame(dataframeDeduplicated_))
})

test_that("Only the deduplicated column gets returned", {
  dataframe_ <- data.frame(
    prefix_1 = c(1, NA, NA),
    prefix_2 = c(NA, 2, NA),
    prefix_3 = c(NA, NA, 3),
    another_col = c(NA, NA, 3))
  
  dataframeDeduplicated_ <- suppressWarnings(
    removeReplicatedColumns(
      dataframe = dataframe_,
      prefix = "prefix"))

  expect_equal(ncol(dataframeDeduplicated_), 2)
})

#' Implement a Slowly Changing Dimension Type 2.
#' 
#' This function implements a Slowly Changing Dimension Type 2 to merge new and
#' current data while maintaining historical records. The function deactivates
#' the old records and activates new ones, ensuring a history-preserving update
#' strategy. Only the changing records are marked as not active and replaced by
#' new active ones.
#' 
#' @param newData `data.frame`. The data frame containing new records.
#' 
#' @param currentData `data.frame`. The data frame containing existing records.
#' 
#' @param key `character` (vector). The columns to be used as key.
#' 
#' @return A combined data frame with old data marked as not active and new data
#'   marked as active.
#' 
#' @importFrom checkmate assert_data_frame assert_vector
#' @importFrom dplyr bind_rows inner_join right_join anti_join
#' 
#' @details
#' The function:
#' 1. Separates active and inactive records from the current data.
#' 2. Gets the old records that are still present in the new data (i.e.,
#'   the ones that can remain active).
#' 3. Gets the records present in new data but not present in still active
#'   current data (i.e., the records to activate) and activates them.
#' 4. Gets the current active records that are not present in the new data
#'   (i.e., the records to deactivate) and deactivates them.
#' 
#' @examples
#' currentData_ <- tibble::tribble(
#'   ~id, ~colA, ~colB, ~colC, ~IS_ACTIVE, ~START_DATE, ~END_DATE,
#'   1, "a1", "b1", "c1", TRUE, Sys.time(), as.Date(NA),
#'   2, "a2", "b2", "c2", TRUE, Sys.time(), as.Date(NA),
#'   3, "a3", "b3", "c3", TRUE, Sys.time(), as.Date(NA))
#' 
#' newData_ <- tibble::tribble(
#'   ~id, ~colA, ~colB, ~colC,
#'   1, "a1", "b1", "c1",
#'   2, "a2", "b2", "c20",
#'   3, "a4", "b4", "c4")
#' 
#' mergedData <- SCD2(newData = newData_, currentData = currentData_)
#' 
#' @export
#' 
SCD2 <- function(newData, currentData, key = names(newData)) {

  assert_data_frame(newData)
  assert_data_frame(currentData)
  assert_vector(key)

  currentInactiveData_ <- currentData |>
    filter(IS_ACTIVE = FALSE)

  currentActiveData_ <- currentData |>
    filter(IS_ACTIVE = TRUE)

  stillActiveData_ <- inner_join(
    unique(currentActiveData_),
    unique(newData),
    by = key)

  newlyActiveData_ <- anti_join(newData, currentActiveData_, by = key)
  newlyActiveData_ <- .activate(dataframe = newlyActiveData_)

  deactivatedData_ <- anti_join(currentActiveData_, newData, by = key)
  deactivatedData_ <- .deactivate(dataframe = deactivatedData_)

  mergedData_ <- bind_rows(
    stillActiveData_,
    newlyActiveData_,
    deactivatedData_,
    currentInactiveData_)

  return(mergedData_)
}

test_that("The new data must be a data frame", {
  expect_error(
    SCD2(
      newData = 1,
      currentData = tibble())
  )
})

test_that("The current data must be a data frame", {
  expect_error(
    SCD2(
      newData = tibble(),
      currentData = 1)
  )
})

test_that("The key must be a vector", {
  expect_error(
    SCD2(
      newData = tibble(),
      currentData = tibble(),
      key = 1)
  )
})

test_that("A data frame must be returned", {
  newData_ <- .activate(dataframe = iris)
  currentData_ <- .deactivate(dataframe = newData_)

  checkmate::expect_data_frame(
    SCD2(newData = newData_, currentData = currentData_))
})

test_that("The START_DATE column is present", {
  newData_ <- .activate(dataframe = iris)
  currentData_ <- .deactivate(dataframe = newData_)

  expect_true(any(names(SCD2(
    newData = newData_,
    currentData = currentData_
  )) == "START_DATE"))
})

test_that("The END_DATE column is present", {
  newData_ <- .activate(dataframe = iris)
  currentData_ <- .deactivate(dataframe = newData_)

  expect_true(any(names(SCD2(
    newData = newData_,
    currentData = currentData_
  )) == "END_DATE"))
})

test_that("The IS_ACTIVE column is present", {
  newData_ <- .activate(dataframe = iris)
  currentData_ <- .deactivate(dataframe = newData_)

  expect_true(any(names(SCD2(
    newData = newData_,
    currentData = currentData_
  )) == "IS_ACTIVE"))
})

test_that("SCD2 gets activated", {
  newData_ <- .activate(dataframe = iris)
  currentData_ <- .deactivate(dataframe = newData_)

  mergedData_ <- SCD2(newData = newData_, currentData = currentData_)

  expect_true(all(unique(mergedData_$IS_ACTIVE) %in% c(TRUE, FALSE)))
})

test_that("The function must output as expected", {
  currentData_ <- tibble::tribble(
    ~id, ~colA, ~colB, ~colC,
    1, "a1", "b1", "c1",
    2, "a2", "b2", "c2",
    3, "a3", "b3", "c3"
  )
  currentData_ <- .activate(dataframe = currentData_)

  newData_ <- tibble::tribble(
    ~id, ~colA, ~colB, ~colC,
    1, "a1", "b1", "c1",  # Identical row.
    2, "a2", "b2", "c20", # Almost identical row.
    3, "a4", "b4", "c4"   # Different row.
  )

  mergedData_ <- SCD2(newData = newData_, currentData = currentData_)

  expectedResult_ <- tibble::tribble(
    ~id, ~colA, ~colB, ~colC, ~IS_ACTIVE,
    1, "a1", "b1", "c1", TRUE,
    2, "a2", "b2", "c20", TRUE,
    3, "a4", "b4", "c4", TRUE,
    2, "a2", "b2", "c2", FALSE,
    3, "a3", "b3", "c3", FALSE) |>
  dplyr::arrange(id)

  arrangedMergedData_ <- mergedData_ |>
    dplyr::select(-ends_with("DATE")) |>
    dplyr::arrange(id)

  expect_equal(arrangedMergedData_, expectedResult_)
})

#' Drop empty lines and columns from the specified data frame.
#' 
#' This function drops all the empty lines and columns from the specified data
#' frame, i.e. all the rows and columns that contain only NAs.
#' 
#' @param dataframe `data.frame`. The data frame from which to remove the empty
#'   lines and columns.
#' 
#' @return The provided data frame without empty lines and columns and all the
#'   types transformed to string.
#' 
#' @importFrom checkmate assert_data_frame
#' @importFrom dplyr filter select where mutate_all
#' 
#' @examples
#' # The first row is going to be dropped.
#' irisTest_ <- iris
#' irisTest_[1, ] <- NA
#' irisTestDropped <- dropEmpty(irisTest_)
#' 
#' # The Species column is going to be dropped.
#' irisTest_ <- iris
#' irisTest_$Test <- NA
#' irisTestDropped <- dropEmpty(irisTest_)
#' 
#' @export
#' 
dropEmpty <- function(dataframe) {

  assert_data_frame(dataframe)

  rowSum_ <- rowSums(is.na(dataframe))

  dataframe <- dataframe |>
    filter(rowSum_ != ncol(dataframe)) |>
    select(where(~ !all(is.na(.)))) |>
    mutate_all(as.character)

  return(dataframe)
}

test_that("The parameter must be a data frame", {
  expect_error(
    dropEmpty(dataframe = 1)
  )
})

test_that("A dataframe must be returned", {
  dataframe_ <- head(iris, 10)

  # The Species column is going to be dropped.
  dataframe_$Species <- NA
  dataframeWithoutSpecies_ <- dropEmpty(dataframe = dataframe_)

  # The first row is going to be dropped.
  dataframe_[1, ] <- NA
  dataframeWithoutFirstRow_ <- dropEmpty(dataframe = dataframe_)

  expect_true(is.data.frame(dataframeWithoutSpecies_))
  expect_true(is.data.frame(dataframeWithoutFirstRow_))
})

test_that("Drops a column", {
  dataframe_ <- head(iris, 10)
  
  # The Species column is going to be dropped.
  dataframe_$Species <- NA
  dataframeWithoutSpecies_ <- dropEmpty(dataframe_)
  numberOfColumns_ <- ncol(dataframeWithoutSpecies_)

  expect_equal(numberOfColumns_, 4)
})

test_that("Drops a row", {
  dataframe_ <- head(iris, 10)

  # The first row is going to be dropped.
  dataframe_[1, ] <- NA
  dataframeWithoutFirstRow_ <- dropEmpty(dataframe = dataframe_)
  numberOfRows_ <- nrow(dataframeWithoutFirstRow_)

  expect_equal(numberOfRows_, 9)
})

core <- c(
  "eppoFindeR",
  "distilleR"
)

.coreUnloaded <- function() {
  search_ <- paste0("package:", core)
  
  return(core[!search_ %in% search()])
}

# Attach the package from the same package library it was loaded from before.
.sameLibrary <- function(pkg) {
  loc_ <- if (pkg %in% loadedNamespaces()) dirname(
    getNamespaceInfo(pkg, "path"))
  
  library(pkg, lib.loc = loc_, character.only = TRUE, warn.conflicts = FALSE)
}

.efsaToolsAttach <- function() {
  toLoad_ <- .coreUnloaded()
  
  suppressPackageStartupMessages(
    lapply(toLoad_, .sameLibrary)
  )
  
  invisible(toLoad_)
}

.efsaToolsAttachMessage <- function(toLoad) {
  if (length(toLoad) == 0) {
    return(NULL)
  }
  
  header_ <- cli::rule(
    left = cli::style_bold("Attaching core EFSATools packages"),
    right = paste0("EFSATools ", .packageVersionStr("EFSATools"))
  )
  
  toLoad <- sort(toLoad)
  versions_ <- vapply(toLoad, .packageVersionStr, character(1))
  
  packages_ <- paste0(
    cli::col_green(cli::symbol$tick),
    ' ',
    cli::col_blue(format(toLoad)),
    ' ',
    cli::ansi_align(versions_, max(cli::ansi_nchar(versions_)))
  )
  
  if (length(packages_) %% 2 == 1) {
    packages_ <- append(packages_, '')
  }
  col1_ <- seq_len(length(packages_) / 2)
  info_ <- paste0(packages_[col1_], "     ", packages_[-col1_])
  
  paste0(header_, '\n', paste(info_, collapse = '\n'))
}

.packageVersionStr <- function(pkg) {
  v_ <- as.character(utils::packageVersion(pkg))
  
  return(v_)
}

test_that("If no packages, shows nothing", {
  expect_null(.efsaToolsAttachMessage(character()))
})

test_that("Runs correctly with an odd number of packages", {
  local_mocked_bindings(.packageVersionStr = function(x) "1.0.0")
  checkmate::expect_character(.efsaToolsAttachMessage(c("a", "b", "c")))
})

#' Activate the records of the specified data frame.
#' 
#' This helper function is used in the context of a Slowly Changing Dimension
#' (SCD) to mark new records of a data frame as active with a start date.
#' 
#' @param dataframe `data.frame`. The data frame to activate.
#' 
#' @return The specified data frame with ACTIVE set to TRUE, START_DATE set to
#'   current time, and END_DATE set to NA.
#'
#' @importFrom checkmate assert_data_frame
#' @importFrom dplyr mutate
#'
#' @keywords internal
#' @noRd
#' 
.activate <- function(dataframe) {

  assert_data_frame(dataframe)
  
  activatedDataframe_ <- dataframe |> mutate(
    IS_ACTIVE = TRUE,
    START_DATE = Sys.time(),
    END_DATE = as.Date(NA)
  )

  return(activatedDataframe_)
}

test_that("The parameter must be a data frame", {
  expect_error(
    .deactivate(dataframe = 1)
  )
})

test_that("Iris gets deactivated", {
  dataframe_ <- .activate(dataframe = iris)
  
  checkmate::expect_data_frame(
    .deactivate(dataframe = dataframe_))
})

test_that("The END_DATE column is present", {
  dataframe_ <- .activate(dataframe = iris)
  dataframe_ <- .deactivate(dataframe = dataframe_)
  
  expect_true(
    any(names(.deactivate(dataframe = dataframe_)) == "END_DATE"))
  expect_true(
    all(!is.null(dataframe_["END_DATE"])))
})

test_that("The IS_ACTIVE column is present", {
  dataframe_ <- .activate(dataframe = iris)
  dataframe_ <- .deactivate(dataframe = dataframe_)
  
  expect_true(
    any(names(.deactivate(dataframe = dataframe_)) == "IS_ACTIVE"))
  expect_true(
    all(dataframe_$IS_ACTIVE == FALSE))
})

test_that("The parameter must be a data frame", {
  expect_error(
    .activate(dataframe = 1)
  )
})

test_that("Iris gets activated", {
  checkmate::expect_data_frame(
    .activate(dataframe = iris))
})

test_that("The START_DATE column is present", {
  dataframe_ <- .activate(dataframe = iris)
  expect_true(
    any(names(dataframe_) == "START_DATE"))
  expect_true(
    all(!is.null(dataframe_$START_DATE)))
})

test_that("The END_DATE column is present", {
  dataframe_ <- .activate(dataframe = iris)
  expect_true(
    any(names(dataframe_) == "END_DATE"))
  expect_true(
    all(is.na(dataframe_$END_DATE)))
})

test_that("The IS_ACTIVE column is present", {
  dataframe_ <- .activate(dataframe = iris)
  expect_true(
    any(names(dataframe_) == "IS_ACTIVE"))
  expect_true(
    all(dataframe_$IS_ACTIVE == TRUE))
})

#' Deactivate the records of the specified data frame.
#' 
#' This helper function is used in the context of a Slowly Changing Dimension
#' (SCD) to mark active records of a data frame as not active with an end date.
#' 
#' @param dataframe `data.frame`. The data frame to deactivate.
#' 
#' @return The specified data frame with IS_ACTIVE set to FALSE and END_DATE set
#'   to current time.
#'
#' @importFrom checkmate assert_data_frame
#' @importFrom dplyr mutate
#'
#' @keywords internal
#' @noRd
#' 
.deactivate <- function(dataframe) {

  assert_data_frame(dataframe)
  
  deactivatedDataframe_ <- dataframe |> mutate(
    IS_ACTIVE = FALSE,
    END_DATE = Sys.time()
  )

  return(deactivatedDataframe_)
}

#' Enrich a data frame with an EFSA catalogue.
#' 
#' This function takes a data frame and joins it with an EFSA catalog. The EFSA
#'   catalog must be itself a data frame.
#' 
#' @param dataframe `data.frame`. The data frame to be enriched.
#' 
#' @param catalogue `data.frame`. The data frame that contains the EFSA
#'   catalogue to be used for the enrichment. It must contain at least two
#'   columns, namely: NAME and CODE.
#' 
#' @param joinBy `character` (string). The variable to be used as the join key.
#' 
#' @param enrichedColumnName `character` (string). The name of the column added
#'   to the original data.
#'
#' @return The specified data frame enriched with the catalogue data.
#' 
#' @importFrom checkmate assert_data_frame assert_names assert_string
#' @importFrom dplyr select mutate left_join
#' @importFrom rlang .data :=
#'
#' @examples
#' dataframe_ <- iris |> dplyr::rename(CODE = Species)
#' 
#' catalogue_ <- iris |>
#'   dplyr::rename(CODE = Species) |>
#'   dplyr::mutate(NAME = "test") |>
#'   dplyr::select(CODE, NAME) |>
#'   unique()
#' 
#' enriched_ <- enrich(
#'   dataframe = dataframe_,
#'   catalogue = catalogue_,
#'   joinBy = "CODE",
#'   enrichedColumnName = "enrichedColumn")
#' 
#' @export
#' 
enrich <- function(dataframe, catalogue, joinBy, enrichedColumnName) {

  assert_data_frame(dataframe)
  assert_data_frame(catalogue)
  assert_names(names(catalogue), must.include = c("NAME", "CODE"))
  assert_string(joinBy)
  assert_string(enrichedColumnName)

  catalogue <- catalogue |>
    select("NAME", "CODE") |>
    mutate(!!joinBy := .data$CODE)

  enriched_ <- left_join(dataframe, catalogue, by = joinBy) |>
    select(!!enrichedColumnName := "NAME", names(dataframe))

  return(enriched_)
}

test_that("The data frame to enrich must be a data frame", {
  expect_error(
    enrich(
      dataframe = 1,
      catalogue = tibble(),
      joinBy = "",
      enrichedColumnName = "")
  )
})

test_that("The catalogue must be a data frame", {
  expect_error(
    enrich(
      dataframe = tibble(),
      catalogue = 1,
      joinBy = "",
      enrichedColumnName = "")
  )
})

test_that("The catalogue must contain the NAME and CODE columns", {
  expect_error(
    enrich(
      dataframe = tibble(),
      catalogue = tibble(),
      joinBy = "",
      enrichedColumnName = "")
  )
})

test_that("The join key must be a string", {
  expect_error(
    enrich(
      dataframe = tibble(),
      catalogue = tibble(),
      joinBy = 1,
      enrichedColumnName = "")
  )
})

test_that("The enriched column name must be a string", {
  expect_error(
    enrich(
      dataframe = tibble(),
      catalogue = tibble(),
      joinBy = "",
      enrichedColumnName = 1)
  )
})

test_that("A data frame must be returned", {
  dataframe_ <- iris |> dplyr::rename(CODE = Species)

  catalogue_ <- iris |>
    dplyr::rename(CODE = Species) |>
    dplyr::mutate(NAME = "test") |>
    dplyr::select(CODE, NAME) |>
    unique()

  enriched_ <- enrich(
    dataframe = dataframe_,
    catalogue = catalogue_,
    joinBy = "CODE",
    enrichedColumnName = "enrichedColumn")

  expect_true(is.data.frame(enriched_))
})

---
title: "EFSATools"
output: rmarkdown::html_vignette
vignette: >
  %\VignetteIndexEntry{EFSATools}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

```{r, include=FALSE}
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>"
)
```

## Overview

The **EFSATools** package brings together all the functions developed for EFSA's ad hoc data collections, providing tools for dataset operations as well as utilities designed to preserve data history.

The package is intended for researchers, analysts, and practitioners who require convenient programmatic access to data collection utilities.

During installation, the following packages developed by EFSA are also installed:
- **eppoFindeR** - [Website](https://openefsa.github.io/eppoFindeR/) | [CRAN](https://cran.r-project.org/package=eppoFindeR).
- **distilleR** - [Website](https://openefsa.github.io/distilleR/) | [CRAN](https://cran.r-project.org/package=distilleR).

These packages are not required to use **EFSATools**, but are included for convenience and can be used directly in the code if needed, for example:

```{r eval=FALSE}
eppoTaxons <- eppoFindeR::taxons()
# and/or
distillerToken <- distilleR::getAuthenticationToken()
```

## Installation

### From CRAN

```{r eval=FALSE}
install.packages("EFSATools")
```

### Development version (from GitHub)

To install the latest development version:

```{r eval=FALSE}
# install.packages("devtools")
devtools::install_github("openefsa/EFSATools")
```

## Basic usage

The main purpose of *EFSATools* is to provide tools for managing datasets and tracking data history within the context of data collections.

Below are examples demonstrating how to use the functions in this package. First, load the *EFSATools* package:

```{r loadLibrary, eval=FALSE}
library(EFSATools)
```

To explore the arguments and usage of a specific function, you can run:

```{r eval=FALSE}
help("<function_name>")
```

This will show the full documentation for the function, including its arguments, return values, and usage examples.

For example, if you are working with the `SCD2()` function, you can check its documentation with:

```{r eval=FALSE}
help("SCD2")
```

## Dropping empty rows and columns from a data frame

If a data frame contain empty rows or columns, you can remove them using the `dropEmpty()` function, as follows:

```{r dropEmpty, eval=FALSE}
irisDropped <- dropEmpty(dataframe = iris)

print(head(irisDropped))
```

## Enriching a data frame with an EFSA's catalogue

The `enrich()` function enables the augmentation of a data frame using information stored in an EFSA's catalogue. It requires specifying the column used to join the two datasets, as well as the name of the column that will contain the enriched information (namely, the 'NAME' field of EFSA's catalogues).

```{r enrich, eval=FALSE}
enrichedDataFrame <- enrich(
  dataframe = dataframe_,
  catalogue = CV_MTX_,
  joinBy = "CODE",
  enrichedColumnName = "enrichedColumn"
)

print(head(enrichedDataFrame))
```

## Removing replicated columns from a data frame

The `removeReplicatedColumns()` function merges all the replicated columns in a data frame into a single column whose name includes the "_deduplicated" suffix. After the merge, the original replicated columns are removed from the data frame.

In the following example, we present a data frame containing the columns *region_1*, *region_2*, ..., *region_n* with *n* > 100. Using the `removeReplicatedColumns()` function, these columns can be efficiently consolidated into a single *region_deduplicated* column, assuming that for each row only one of the *n* columns contains a meaningful (non-NA) value.

```{r removeReplicatedColumns, eval=FALSE}
iris$Species_1 <- iris$Species
iris$Species_2 <- iris$Species
iris$Species <- NULL

irisDeduplicated <- removeReplicatedColumns(
  dataframe = iris,
  prefix = "Species_"
)

print(head(irisDeduplicated))
```

## Implementing a "Simple" Slowly Changing Dimension Type 2 (SSCD2)

The `SSCD2()` function makes it possible to preserve data history when new data becomes available by implementing a simplified version of Slowly Changing Dimension Type 2. It marks all records in the current data frame as inactive and appends the new data, flagging each newly added record as active.

Unlike the `SCD2()` function, `SSCD2()` does not check which records have actually changed. Instead, it marks all existing records as inactive and treats all incoming records as new, setting the previous ones to inactive status even if they are still included in the updated dataset.

An example of how to use the function is provided below:

```{r SSCD2, eval=FALSE}
sscd2Dataframe <- SSCD2(newData = newDataframe, currentData = oldDataframe)

print(head(sscd2Dataframe))
```

## Implementing a Slowly Changing Dimension Type 2 (SCD2)

The `SCD2()` function makes it possible to preserve data history when new data becomes available by implementing a Slowly Changing Dimension Type 2. It compares the current records with the new ones, marking as inactive any existing records that no longer appear in the updated dataset. Then, it flags as active any new records that are not present among the currently active data.

Unlike the `SSCD2()` function, `SCD2()` checks which records have actually changed. It marks as inactive any existing records that no longer appear in the updated dataset, and flags as active any new records that are not present among the currently active data.

An example of how to use the function is provided below:

```{r SCD2, eval=FALSE}
scd2Dataframe <- SCD2(newData = newDataframe, currentData = oldDataframe)

print(head(scd2Dataframe))
```

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/enrich.R
\name{enrich}
\alias{enrich}
\title{Enrich a data frame with an EFSA catalogue.}
\usage{
enrich(dataframe, catalogue, joinBy, enrichedColumnName)
}
\arguments{
\item{dataframe}{\code{data.frame}. The data frame to be enriched.}

\item{catalogue}{\code{data.frame}. The data frame that contains the EFSA
catalogue to be used for the enrichment. It must contain at least two
columns, namely: NAME and CODE.}

\item{joinBy}{\code{character} (string). The variable to be used as the join key.}

\item{enrichedColumnName}{\code{character} (string). The name of the column added
to the original data.}
}
\value{
The specified data frame enriched with the catalogue data.
}
\description{
This function takes a data frame and joins it with an EFSA catalog. The EFSA
catalog must be itself a data frame.
}
\examples{
dataframe_ <- iris |> dplyr::rename(CODE = Species)

catalogue_ <- iris |>
  dplyr::rename(CODE = Species) |>
  dplyr::mutate(NAME = "test") |>
  dplyr::select(CODE, NAME) |>
  unique()

enriched_ <- enrich(
  dataframe = dataframe_,
  catalogue = catalogue_,
  joinBy = "CODE",
  enrichedColumnName = "enrichedColumn")

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/SCD2.R
\name{SCD2}
\alias{SCD2}
\title{Implement a Slowly Changing Dimension Type 2.}
\usage{
SCD2(newData, currentData, key = names(newData))
}
\arguments{
\item{newData}{\code{data.frame}. The data frame containing new records.}

\item{currentData}{\code{data.frame}. The data frame containing existing records.}

\item{key}{\code{character} (vector). The columns to be used as key.}
}
\value{
A combined data frame with old data marked as not active and new data
marked as active.
}
\description{
This function implements a Slowly Changing Dimension Type 2 to merge new and
current data while maintaining historical records. The function deactivates
the old records and activates new ones, ensuring a history-preserving update
strategy. Only the changing records are marked as not active and replaced by
new active ones.
}
\details{
The function:
\enumerate{
\item Separates active and inactive records from the current data.
\item Gets the old records that are still present in the new data (i.e.,
the ones that can remain active).
\item Gets the records present in new data but not present in still active
current data (i.e., the records to activate) and activates them.
\item Gets the current active records that are not present in the new data
(i.e., the records to deactivate) and deactivates them.
}
}
\examples{
currentData_ <- tibble::tribble(
  ~id, ~colA, ~colB, ~colC, ~IS_ACTIVE, ~START_DATE, ~END_DATE,
  1, "a1", "b1", "c1", TRUE, Sys.time(), as.Date(NA),
  2, "a2", "b2", "c2", TRUE, Sys.time(), as.Date(NA),
  3, "a3", "b3", "c3", TRUE, Sys.time(), as.Date(NA))

newData_ <- tibble::tribble(
  ~id, ~colA, ~colB, ~colC,
  1, "a1", "b1", "c1",
  2, "a2", "b2", "c20",
  3, "a4", "b4", "c4")

mergedData <- SCD2(newData = newData_, currentData = currentData_)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/SSCD2.R
\name{SSCD2}
\alias{SSCD2}
\title{Implement a "Simple" Slowly Changing Dimension Type 2.}
\usage{
SSCD2(newData, currentData)
}
\arguments{
\item{newData}{\code{data.frame}. The data frame containing new records.}

\item{currentData}{\code{data.frame}. The data frame containing existing records.}
}
\value{
A combined data frame with all old data marked as not active and new
data marked as active.
}
\description{
This function implements a Simplified version of Slowly Changing Dimension
Type 2 to merge new and current data while maintaining historical records.
The function deactivates all the old records and activates new ones, ensuring
a history-preserving update strategy. The difference between a standard SCD2
is that this simplified version applies no checks on the data, deactivating
all the old records and activating the new ones, even if some of the old
records are still active.
}
\examples{
currentData_ <- tibble::tribble(
  ~id, ~colA, ~colB, ~colC, ~IS_ACTIVE, ~START_DATE, ~END_DATE,
  1, "a1", "b1", "c1", TRUE, Sys.time(), as.Date(NA),
  2, "a2", "b2", "c2", TRUE, Sys.time(), as.Date(NA),
  3, "a3", "b3", "c3", TRUE, Sys.time(), as.Date(NA))

newData_ <- tibble::tribble(
  ~id, ~colA, ~colB, ~colC,
  1, "a1", "b1", "c1",
  2, "a2", "b2", "c20",
  3, "a4", "b4", "c4")

mergedData <- SSCD2(newData = newData_, currentData = currentData_)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/removeReplicatedColumns.R
\name{removeReplicatedColumns}
\alias{removeReplicatedColumns}
\title{Drop and merge replicated columns from the specified data frame.}
\usage{
removeReplicatedColumns(dataframe, prefix)
}
\arguments{
\item{dataframe}{\code{data.frame}. The data frame from which to drop the
replicated columns.}

\item{prefix}{\code{character} (string). The prefix with which the name of the
replicated columns starts.}
}
\value{
The specified data frame with an additional deduplicated column and
all the types transformed to string.
}
\description{
This function drops and merges all the replicated columns from the specified
data frame.
}
\details{
All the occurrences of "N/A", "NA", and empty strings (case insensitive)
inside the provided data frame are replaced with NAs of type character.
Then, all and only the columns starting with the specified prefix are
selected and united into a single column with name ending per
"_deduplicated". All empty entries in the new deduplicated column are
replaced with NAs. Finally, the new column is bound with the other columns of
the initial dataframe.
}
\examples{
irisTest_ <- iris
irisTest_$Species_1 <- irisTest_$Species
irisTest_$Species_2 <- irisTest_$Species
irisTest_$Species <- NULL

deduplicatedDataframe_ <- removeReplicatedColumns(
  dataframe = irisTest_,
  prefix = "Species_")

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/EFSATools-package.R
\docType{package}
\name{EFSATools-package}
\alias{EFSATools}
\alias{EFSATools-package}
\title{EFSATools: EFSA Ensemble of Data Collections Tools}
\description{
\if{html}{\figure{logo.png}{options: style='float: right' alt='logo' width='120'}}

Provides tools for dataset operations and utilities designed to preserve data history within EFSA's ad hoc data collections. It also imports packages developed by EFSA that provide additional support for data collection activities.
}
\seealso{
Useful links:
\itemize{
  \item \url{https://openefsa.github.io/EFSATools/}
  \item Report bugs at \url{https://github.com/openefsa/EFSATools/issues}
}

}
\author{
\strong{Maintainer}: Luca Belmonte \email{luca.belmonte@efsa.europa.eu} (\href{https://orcid.org/0000-0002-7977-9170}{ORCID})

Authors:
\itemize{
  \item Lorenzo Copelli (\href{https://orcid.org/0009-0002-4305-065X}{ORCID})
}

}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/dropEmpty.R
\name{dropEmpty}
\alias{dropEmpty}
\title{Drop empty lines and columns from the specified data frame.}
\usage{
dropEmpty(dataframe)
}
\arguments{
\item{dataframe}{\code{data.frame}. The data frame from which to remove the empty
lines and columns.}
}
\value{
The provided data frame without empty lines and columns and all the
types transformed to string.
}
\description{
This function drops all the empty lines and columns from the specified data
frame, i.e. all the rows and columns that contain only NAs.
}
\examples{
# The first row is going to be dropped.
irisTest_ <- iris
irisTest_[1, ] <- NA
irisTestDropped <- dropEmpty(irisTest_)

# The Species column is going to be dropped.
irisTest_ <- iris
irisTest_$Test <- NA
irisTestDropped <- dropEmpty(irisTest_)

}
